import { createVNode, render, isVNode } from 'vue'
import type { VNode, ComponentPublicInstance, AppContext } from 'vue'
import MessageConstructor from './src/index.vue'
import { isFunction, isString } from '../utils/types'
import { messageTypes } from './message'

let seed = 1;
let zIndex = 1000;
const instances: any = []

const message: any = (options: any = {}, context?: AppContext | null) => {
	const appendTo: HTMLElement | null = document.body;
	const container = document.createElement('div')
	zIndex++;

	let verticalOffset = options.offset || 0
	instances.forEach(({ vm }: any) => {
		verticalOffset += (vm.el?.offsetHeight || 0) + 16
	})
	verticalOffset += 16

	const userOnClose = options.onClose
	const id = `message_${seed++}`;

	const props = {
		zIndex: zIndex,
		duration: 2000,
		...options,
		offset: verticalOffset,
		id,
		onClose: () => {
			close(id, userOnClose)
		},
	}

	const messageContent = props.message

	const vm = createVNode(
		MessageConstructor,
		props,
		isFunction(messageContent)
			? { default: messageContent }
			: isVNode(messageContent)
				? { default: () => messageContent }
				: null
	)

	vm.props!.onDestroy = () => {
		render(null, container)
	}

	render(vm, container)
	instances.push({ vm })
	appendTo.appendChild(container.firstElementChild!)

	return {
		// instead of calling the onClose function directly, setting this value so that we can have the full lifecycle
		// for out component, so that all closing steps will not be skipped.
		close: () =>
		((
			vm.component!.proxy as ComponentPublicInstance<{ visible: boolean }>
		).visible = false),
	}

}

messageTypes.forEach((t) => {
	message[t] = (options = {}, appContext?: AppContext | null) => {
		if (isString(options) || isVNode(options)) {
			options = {
				message: options,
			}
		}
		return message(
			{
				...options,
				type: t,
			},
			appContext
		)
	}
})

function close(id: string, userOnClose?: (vm: VNode) => void): void {
	const idx = instances.findIndex(({ vm }: any) => id === vm?.props.id)
	if (idx === -1) return

	const { vm } = instances[idx]
	if (!vm) return

	userOnClose?.(vm)

	const removedHeight = vm.el!.offsetHeight
	instances.splice(idx, 1)

	const len = instances.length
	if (len < 1) return
	for (let i = idx; i < len; i++) {
		const pos =
			Number.parseInt(instances[i].vm.el!.style['top'], 10) - removedHeight - 16
		instances[i].vm.component!.props.offset = pos
	}
}

export function closeAll(): void {
	for (let i = instances.length - 1; i >= 0; i--) {
		const instance = instances[i].vm.component
			; (instance?.proxy as any)?.close()
	}
}

message.closeAll = closeAll
message._context = null

export default message