import type { VNode, AppContext } from 'vue'

export type MessageProps = {
  type: String;
  zIndex: Number;
  duration: Number;
  message: String | Object | Function;
  offset: Number;
  onClose: Function;
  dangerouslyUseHTMLString: {
    type: Boolean,
    default: false,
  },
}

type MessageOptions = Omit<MessageProps, 'id'> & {
  appendTo?: HTMLElement | string
}

type MessageOptionsTyped = Omit<MessageOptions, 'type'>

type MessageParams = Partial<MessageOptions> | string | VNode
type MessageParamsTyped = Partial<MessageOptionsTyped> | string | VNode

interface MessageHandle {
  close: () => void
}

type MessageFn = ((
  options?: MessageParams,
  appContext?: null | AppContext
) => MessageHandle) & {
  closeAll(): void
}

export type MessageTypedFn = (
  options?: MessageParamsTyped,
  appContext?: null | AppContext
) => MessageHandle

export interface Message extends MessageFn {
  success: MessageTypedFn
  warning: MessageTypedFn
  info: MessageTypedFn
  error: MessageTypedFn
}

export const messageTypes = ['success', 'info', 'warning', 'error'] as const