
import {
	Comment,
	Fragment,
	Text,
	defineComponent,
	cloneVNode,
	withDirectives,
	h
} from "vue";
import type { Ref, VNode } from 'vue'
import { NOOP, isObject } from '@vue/shared'

const NAME = 'ShOnlyChild'

const OnlyChild = defineComponent({
	name: NAME,
	emits: [],
	components: {},
	props: {},
	setup(props, { slots, attrs }) {
		return () => {
			const defaultSlot = slots.default?.(attrs)
			const firstLegitNode = findFirstLegitChild(defaultSlot)
			if (!firstLegitNode) {
				console.error(NAME, 'no valid child node found')
				return null
			}
			// return withDirectives(cloneVNode(firstLegitNode!, attrs), [])
			return cloneVNode(firstLegitNode!, attrs)
		}
	},
});

function findFirstLegitChild(node: VNode[] | undefined): VNode | null {
	if (!node) return null
	const children = node as VNode[]
	for (const child of children) {
		if (isObject(child)) {
			switch (child.type) {
				case Comment:
					continue
				case Text:
					return wrapTextContent(child)
				case 'svg':
					return wrapTextContent(child)
				case Fragment:
					return findFirstLegitChild(child.children as VNode[])
				default:
					return child
			}
		}
		return wrapTextContent(child)
	}
	return null
}

function wrapTextContent(s: string | VNode) {
	return h('span', { clasd: 'sh-only-child__content' }, s)
}

export type OnlyChildExpose = {
	forwardRef: Ref<HTMLElement>
}

export default OnlyChild