import pilotConstructor from './src/index.vue'
import type { ComponentPublicInstance } from 'vue'
import { createVNode, render } from 'vue'

let pilotObj = [] as any[];

const pilot = (options: any) => {
  const container = document.createElement('div');

  const onClosed = () => {
    render(null, container);
    container.remove()
    pilotObj = []
  }
  const vnode = createVNode(
    pilotConstructor,
    { ...options, onClosed }
  )

  render(vnode, container);

  const appendTo = document.body;
  appendTo.appendChild(container.firstElementChild as Node)

  const instance = vnode.component;
  const vm = instance?.proxy as ComponentPublicInstance<{ close: () => void }>;
  return vm;
}



const pilotGenerate = (options: any) => {
  if (pilotObj.length === 0) {
    const _pilot = pilot(options)
    pilotObj.push(_pilot)
  }
  return pilotObj[0]
}



interface Content {
  element: string,
  html: string,
  placement?:
  | "auto"
  | "bottom-start"
  | "auto-start"
  | "auto-end"
  | "top"
  | "bottom"
  | "right"
  | "left"
  | "top-start"
  | "top-end"
  | "bottom-end"
  | "right-start"
  | "right-end"
  | "left-start"
  | "left-end";
}

interface Step {
  current: number,
  total: number,
}

interface Action {
  preStep?: () => void;
  finish?: () => void;
  cancle?: () => void;
}


interface PilotjsConfig {
  content: Content,
  step: Step
  callback?: Action
}

class PilotJs {
  instance: any;
  currentStep: string = '';
  offset: number = 0;
  create() {
    if (!this.instance) {
      this.instance = pilotGenerate({});
    }
  }
  show(config: PilotjsConfig) {
    if (this.instance) {
      const { content, step, callback } = config;
      this.instance.position(content, step, callback)
    }
  }
  hide() {
    this.instance && this.instance.close();
  }
  destroy() {
    this.instance && this.instance.destroy();
    this.instance = null;
  }
}


const pilotInstance = {
  install(app: any) {
    app.config.globalProperties.$pilot = new PilotJs()
  }
}

export default pilotInstance



